@@ -8,10 +8,14 @@
 #include <time.h>
 #include <gdk/gdkx.h>
 
+#include <glib/gi18n.h>
+
 #include <glibtop.h>
 #include <glibtop/cpu.h>
 #include <glibtop/mem.h>
 #include <glibtop/swap.h>
+#include <glibtop/netload.h>
+#include <glibtop/netlist.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 
@@ -20,8 +24,48 @@
 #include "util.h"
 
 
+struct _LoadGraph {
+
+	guint n;
+	gint type;
+	guint speed;
+	guint draw_width, draw_height;
+	guint num_points;
+	guint num_cpus;
+
+	guint allocated;
+
+	GdkColor *colors;
+	gfloat **data;
+	guint data_size;
+	guint *pos;
+
+	gboolean colors_allocated;
+	GtkWidget *main_widget;
+	GtkWidget *disp;
+
+	GdkPixmap *pixmap;
+	GdkGC *gc;
+	guint timer_index;
+
+	gboolean draw;
+
+	LoadGraphLabels labels;
+
+	guint64 cpu_time [GLIBTOP_NCPU] [NCPUSTATES];
+	guint64 cpu_last [GLIBTOP_NCPU] [NCPUSTATES];
+	gboolean cpu_initialized;
+
+	guint64 net_last_in, net_last_out;
+	float net_max;
+	GTimeVal net_time;
+};
+
+
 #define FRAME_WIDTH 4
 
+
+
 /* Redraws the backing pixmap for the load graph and updates the window */
 void
 load_graph_draw (LoadGraph *g)
@@ -56,7 +100,7 @@ load_graph_draw (LoadGraph *g)
 			gdk_color_alloc (colormap, &(g->colors [i]));
 		}
 
-		g->colors_allocated = 1;
+		g->colors_allocated = TRUE;
 	}
 
 /* Erase Rectangle */
@@ -181,7 +225,7 @@ get_load (gfloat data [2], LoadGraph *g)
 		load = CLAMP(100.0f*data[i], 0.0f, 100.0f);
 
 		text = g_strdup_printf ("%.1f%%", load);
-		gtk_label_set_text (GTK_LABEL (g->cpu_labels[i]), text);
+		gtk_label_set_text (GTK_LABEL (g->labels.cpu[i]), text);
 		g_free (text);
 	}
 }
@@ -205,9 +249,9 @@ get_memory (gfloat data [1], LoadGraph *g)
 	text1 = SI_gnome_vfs_format_file_size_for_display (mem.total);
 	text2 = SI_gnome_vfs_format_file_size_for_display (mem.user);
 	text3 = g_strdup_printf ("  %.1f %%", mempercent * 100.0f);
-	gtk_label_set_text (GTK_LABEL (g->memused_label), text2);
-	gtk_label_set_text (GTK_LABEL (g->memtotal_label), text1);
-	gtk_label_set_text (GTK_LABEL (g->mempercent_label), text3);
+	gtk_label_set_text (GTK_LABEL (g->labels.memused), text2);
+	gtk_label_set_text (GTK_LABEL (g->labels.memtotal), text1);
+	gtk_label_set_text (GTK_LABEL (g->labels.mempercent), text3);
 	g_free (text1);
 	g_free (text2);
 	g_free (text3);
@@ -215,9 +259,9 @@ get_memory (gfloat data [1], LoadGraph *g)
 	text1 = SI_gnome_vfs_format_file_size_for_display (swap.total);
 	text2 = SI_gnome_vfs_format_file_size_for_display (swap.used);
 	text3 = g_strdup_printf ("  %.1f %%", swappercent * 100.0f);
-	gtk_label_set_text (GTK_LABEL (g->swapused_label), text2);
-	gtk_label_set_text (GTK_LABEL (g->swaptotal_label), text1);
-	gtk_label_set_text (GTK_LABEL (g->swappercent_label), text3);
+	gtk_label_set_text (GTK_LABEL (g->labels.swapused), text2);
+	gtk_label_set_text (GTK_LABEL (g->labels.swaptotal), text1);
+	gtk_label_set_text (GTK_LABEL (g->labels.swappercent), text3);
 	g_free (text1);
 	g_free (text2);
 	g_free (text3);
@@ -226,6 +270,102 @@ get_memory (gfloat data [1], LoadGraph *g)
 	data [1] = swappercent;
 }
 
+static void
+net_scale (LoadGraph *g, float new_max)
+{
+	gint i;
+	float old_max = MAX (g->net_max, 1.0f), scale;
+
+	if (new_max <= old_max)
+		return;
+
+	scale = old_max / new_max;
+
+	for (i = 0; i < g->num_points; i++) {
+		if (g->data[i][0] >= 0.0f) {
+			g->data[i][0] *= scale;
+			g->data[i][1] *= scale;
+		}
+	}
+
+	g->net_max = new_max;
+}
+
+static void
+get_net (float data [2], LoadGraph *g)
+{
+	glibtop_netlist netlist;
+	char **ifnames;
+	guint32 i;
+	guint64 in = 0, out = 0;
+	GTimeVal time;
+	float din, dout;
+	gchar *text1, *text2;
+
+	ifnames = glibtop_get_netlist(&netlist);
+
+	for (i = 0; i < netlist.number; ++i)
+	{
+		glibtop_netload netload;
+		glibtop_get_netload (&netload, ifnames[i]);
+
+		if (netload.if_flags & (1 << GLIBTOP_IF_FLAGS_LOOPBACK))
+			continue;
+
+		/* Don't skip interfaces that are down (GLIBTOP_IF_FLAGS_UP)
+		   to avoid spikes when they are brought up */
+
+		in  += netload.bytes_in;
+		out += netload.bytes_out;
+	}
+
+	g_strfreev(ifnames);
+
+	g_get_current_time (&time);
+
+	if (in >= g->net_last_in && out >= g->net_last_out &&
+	    g->net_time.tv_sec != 0) {
+		float dtime;
+		dtime = time.tv_sec - g->net_time.tv_sec +
+			(float) (time.tv_usec - g->net_time.tv_usec) / G_USEC_PER_SEC;
+		din   = (in - g->net_last_in) / dtime;
+		dout  = (out - g->net_last_out) / dtime;
+	} else {
+		/* Don't calc anything if new data is less than old (interface
+		   removed, counters reset, ...) or if it is the first time */
+		din  = 0.0f;
+		dout = 0.0f;
+	}
+
+	g->net_last_in  = in;
+	g->net_last_out = out;
+	g->net_time     = time;
+
+	data [0] = din / MAX (g->net_max, 1.0f);
+	data [1] = dout / MAX (g->net_max, 1.0f);
+
+	net_scale (g, MAX (din, dout));
+
+	text1 = SI_gnome_vfs_format_file_size_for_display (din);
+	text2 = g_strdup_printf (_("%s/s"), text1);
+	gtk_label_set_text (GTK_LABEL (g->labels.net_in), text2);
+	g_free (text1);
+	g_free (text2);
+
+	text1 = SI_gnome_vfs_format_file_size_for_display (in);
+	gtk_label_set_text (GTK_LABEL (g->labels.net_in_total), text1);
+	g_free (text1);
+
+	text1 = SI_gnome_vfs_format_file_size_for_display (dout);
+	text2 = g_strdup_printf (_("%s/s"), text1);
+	gtk_label_set_text (GTK_LABEL (g->labels.net_out), text2);
+	g_free (text1);
+	g_free (text2);
+
+	text1 = SI_gnome_vfs_format_file_size_for_display (out);
+	gtk_label_set_text (GTK_LABEL (g->labels.net_out_total), text1);
+	g_free (text1);
+}
 
 
 /*
@@ -258,9 +398,11 @@ shift_right(LoadGraph *g)
 
 
 /* Updates the load graph when the timeout expires */
-static int
-load_graph_update (LoadGraph *g)
+static gboolean
+load_graph_update (gpointer user_data)
 {
+	LoadGraph * const g = user_data;
+
 	shift_right(g);
 
 	switch (g->type) {
@@ -270,6 +412,9 @@ load_graph_update (LoadGraph *g)
 	case LOAD_GRAPH_MEM:
 		get_memory (g->data [0], g);
 		break;
+	case LOAD_GRAPH_NET:
+		get_net (g->data[0], g);
+		break;
 	}
 
 	if (g->draw)
@@ -283,8 +428,7 @@ load_graph_unalloc (LoadGraph *g)
 {
 	int i;
 
-	if (!g->allocated)
-		return;
+	g_return_if_fail(g->allocated);
 
 	for (i = 0; i < g->num_points; i++) {
 		g_free (g->data [i]);
@@ -309,8 +453,7 @@ load_graph_alloc (LoadGraph *g)
 {
 	int i, j;
 
-	if (g->allocated)
-		return;
+	g_return_if_fail(!g->allocated);
 
 	g->data = g_new (gfloat *, g->num_points);
 	g->pos = g_new0 (guint, g->num_points);
@@ -326,7 +469,8 @@ load_graph_alloc (LoadGraph *g)
 	g->allocated = TRUE;
 }
 
-static gint
+
+static gboolean
 load_graph_configure (GtkWidget *widget, GdkEventConfigure *event,
 		      gpointer data_ptr)
 {
@@ -334,14 +478,12 @@ load_graph_configure (GtkWidget *widget, GdkEventConfigure *event,
 
 	if (c->pixmap) {
 		gdk_pixmap_unref (c->pixmap);
-		c->pixmap = NULL;
 	}
 
-	if (!c->pixmap)
-		c->pixmap = gdk_pixmap_new (widget->window,
-					    widget->allocation.width,
-					    widget->allocation.height,
-					    gtk_widget_get_visual (c->disp)->depth);
+	c->pixmap = gdk_pixmap_new (widget->window,
+				    widget->allocation.width,
+				    widget->allocation.height,
+				    gtk_widget_get_visual (c->disp)->depth);
 
 	gdk_draw_rectangle (c->pixmap,
 			    widget->style->black_gc,
@@ -359,10 +501,10 @@ load_graph_configure (GtkWidget *widget, GdkEventConfigure *event,
 	load_graph_draw (c);
 
 	return TRUE;
-	event = NULL;
 }
 
-static gint
+
+static gboolean
 load_graph_expose (GtkWidget *widget, GdkEventExpose *event,
 		   gpointer data_ptr)
 {
@@ -377,20 +519,23 @@ load_graph_expose (GtkWidget *widget, GdkEventExpose *event,
 	return FALSE;
 }
 
-static void
+
+static gboolean
 load_graph_destroy (GtkWidget *widget, gpointer data_ptr)
 {
 	LoadGraph * const g = data_ptr;
 
 	load_graph_stop (g);
 
-	if (g->timer_index != -1)
-		gtk_timeout_remove (g->timer_index);
+	if (g->timer_index)
+		g_source_remove (g->timer_index);
 
-	return;
-	widget = NULL;
+	load_graph_unalloc(g);
+
+	return FALSE;
 }
 
+
 LoadGraph *
 load_graph_new (gint type, ProcData *procdata)
 {
@@ -406,9 +551,28 @@ load_graph_new (gint type, ProcData *procdata)
 			g->n = 1;
 		else
 			g->n = procdata->config.num_cpus;
+
+		for(i = 0; i < G_N_ELEMENTS(g->labels.cpu); ++i)
+			g->labels.cpu[i] = gtk_label_new(NULL);
+
 		break;
+
 	case LOAD_GRAPH_MEM:
 		g->n = 2;
+		g->labels.memused = gtk_label_new(NULL);
+		g->labels.memtotal = gtk_label_new(NULL);
+		g->labels.mempercent = gtk_label_new(NULL);
+		g->labels.swapused = gtk_label_new(NULL);
+		g->labels.swaptotal = gtk_label_new(NULL);
+		g->labels.swappercent = gtk_label_new(NULL);
+		break;
+
+	case LOAD_GRAPH_NET:
+		g->n = 2;
+		g->labels.net_in = gtk_label_new(NULL);
+		g->labels.net_in_total = gtk_label_new(NULL);
+		g->labels.net_out = gtk_label_new(NULL);
+		g->labels.net_out_total = gtk_label_new(NULL);
 		break;
 	}
 
@@ -428,9 +592,13 @@ load_graph_new (gint type, ProcData *procdata)
 		g->colors[2] = procdata->config.mem_color;
 		g->colors[3] = procdata->config.swap_color;
 		break;
+	case LOAD_GRAPH_NET:
+		g->colors[2] = procdata->config.net_in_color;
+		g->colors[3] = procdata->config.net_out_color;
+		break;
 	}
 
-	g->timer_index = -1;
+	g->timer_index = 0;
 
 	g->draw = FALSE;
 
@@ -454,29 +622,76 @@ load_graph_new (gint type, ProcData *procdata)
 
 	gtk_widget_show_all (g->main_widget);
 
-	g->timer_index = gtk_timeout_add (g->speed,
-					  (GtkFunction) load_graph_update, g);
-
 	return g;
 }
 
+
 void
 load_graph_start (LoadGraph *g)
 {
-	if (!g)
-		return;
+	if(!g->timer_index) {
 
-	if (g->timer_index == -1)
-		g->timer_index = gtk_timeout_add (g->speed,
-						  (GtkFunction) load_graph_update, g);
+		load_graph_update(g);
+
+		g->timer_index = g_timeout_add (g->speed,
+						load_graph_update,
+						g);
+	}
 
 	g->draw = TRUE;
 }
 
+
 void
 load_graph_stop (LoadGraph *g)
 {
-	if (!g)
-		return;
+	/* don't draw anymore, but continue to poll */
 	g->draw = FALSE;
 }
+
+
+
+void
+load_graph_change_speed (LoadGraph *g,
+			 guint new_speed)
+{
+	g->speed = new_speed;
+
+	if(g->timer_index) {
+		g_source_remove (g->timer_index);
+		g->timer_index = g_timeout_add (g->speed,
+						load_graph_update,
+						g);
+	}
+}
+
+
+
+GdkColor*
+load_graph_get_colors (LoadGraph *g)
+{
+	return g->colors;
+}
+
+
+
+void
+load_graph_reset_colors (LoadGraph *g)
+{
+	g->colors_allocated = FALSE;
+}
+
+
+
+LoadGraphLabels*
+load_graph_get_labels (LoadGraph *g)
+{
+	return &g->labels;
+}
+
+
+GtkWidget*
+load_graph_get_widget (LoadGraph *g)
+{
+	return g->main_widget;
+}
