@@ -20,18 +20,29 @@
 #define _PROCDIALOGS_H_
 
 
-#include <gnome.h>
-#include <procman.h>
+#include <glib.h>
+#include "procman.h"
+
+/* These are the actual range of settable values. Values outside this range
+   are scaled back to these limits. So show these limits in the slider
+*/
+#ifdef linux
+#define RENICE_VAL_MIN -20
+#define RENICE_VAL_MAX 19
+#else /* ! linux */
+#define RENICE_VAL_MIN -20
+#define RENICE_VAL_MAX 20
+#endif
+
+
 
-extern int kill_signal G_GNUC_INTERNAL;
 
 void			procdialog_create_hide_dialog (ProcData *data) G_GNUC_INTERNAL;
 void			procdialog_create_kill_dialog (ProcData *data, int signal) G_GNUC_INTERNAL;
 void 			procdialog_create_renice_dialog (ProcData *data) G_GNUC_INTERNAL;
-void			procdialog_create_root_password_dialog (gint type, 
+gboolean		procdialog_create_root_password_dialog (gint type,
 								ProcData *procdata,
-								gint pid, gint extra_value,
-								gchar *text) G_GNUC_INTERNAL;
+								gint pid, gint extra_value) G_GNUC_INTERNAL;
 void 			procdialog_create_memmaps_dialog (ProcData *data) G_GNUC_INTERNAL;
 void			procdialog_create_preferences_dialog (ProcData *data) G_GNUC_INTERNAL;
 
