@@ -69,25 +69,28 @@ sort_ints (GtkTreeModel *model, GtkTreeIter *itera, GtkTreeIter *iterb, gpointer
 
 	switch (col) {
 	case COL_MEM:
-		return PROCMAN_CMP(infoa->mem, infob->mem);
+		return PROCMAN_RCMP(infoa->mem, infob->mem);
 
 	case COL_VMSIZE:
-		return PROCMAN_CMP(infoa->vmsize, infob->vmsize);
+		return PROCMAN_RCMP(infoa->vmsize, infob->vmsize);
 
 	case COL_MEMRES:
-		return PROCMAN_CMP(infoa->memres, infob->memres);
+		return PROCMAN_RCMP(infoa->memres, infob->memres);
 
 	case COL_MEMSHARED:
-		return PROCMAN_CMP(infoa->memshared, infob->memshared);
+		return PROCMAN_RCMP(infoa->memshared, infob->memshared);
 
 	case COL_MEMRSS:
-		return PROCMAN_CMP(infoa->memrss, infob->memrss);
+		return PROCMAN_RCMP(infoa->memrss, infob->memrss);
 
 	case COL_MEMXSERVER:
-		return PROCMAN_CMP(infoa->memxserver, infob->memxserver);
+		return PROCMAN_RCMP(infoa->memxserver, infob->memxserver);
 
 	case COL_CPU_TIME:
-		return PROCMAN_CMP(infoa->cpu_time_last, infob->cpu_time_last);
+		return PROCMAN_RCMP(infoa->cpu_time_last, infob->cpu_time_last);
+
+	case COL_CPU:
+		return PROCMAN_RCMP(infoa->pcpu, infob->pcpu);
 
 	default:
 		g_assert_not_reached();
@@ -240,7 +243,7 @@ proctable_new (ProcData * const procdata)
 		N_("User"),
 		N_("Status"),
 		N_("Memory"),
-		N_("VM Size"),
+		N_("Virtual Memory"),
 		N_("Resident Memory"),
 		N_("Shared Memory"),
 		N_("RSS Memory"),
@@ -362,6 +365,7 @@ proctable_new (ProcData * const procdata)
 		case COL_MEMRSS:
 		case COL_MEMXSERVER:
 		case COL_CPU_TIME:
+		case COL_CPU:
 			gtk_tree_sortable_set_sort_func (
 				GTK_TREE_SORTABLE (model),
 				i,
@@ -881,7 +885,7 @@ refresh_list (ProcData *procdata, const unsigned *pid_list, const guint n)
 	/* use a hash for fast lookup
 	   !NULL as data for every key */
 
-	pid_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
+	pid_hash = g_hash_table_new(NULL, NULL);
 	for(i = 0; i < n; ++i)
 		g_hash_table_insert(pid_hash,
 				    GINT_TO_POINTER(pid_list[i]),
@@ -944,7 +948,7 @@ proctable_update_list (ProcData * const procdata)
 	/* FIXME: total cpu time elapsed should be calculated on an individual basis here
 	** should probably have a total_time_last gint in the ProcInfo structure */
 	glibtop_get_cpu (&cpu);
-	total_time = cpu.total - total_time_last;
+	total_time = MAX(cpu.total - total_time_last, 1);
 	total_time_last = cpu.total;
 
 	refresh_list (procdata, pid_list, proclist.number);
@@ -1002,7 +1006,7 @@ proctable_free_table (ProcData * const procdata)
 	procdata->info = NULL;
 
 	g_hash_table_destroy(procdata->pids);
-	procdata->pids = g_hash_table_new(g_direct_hash, g_direct_equal);
+	procdata->pids = g_hash_table_new(NULL, NULL);
 }
 
 
